#!/usr/bin/env python2
# -*- coding: utf-8 -*-

from psychopy import visual, core, gui, data, event, parallel
from psychopy.tools.filetools import fromFile, toFile
import numpy as np
import sounddevice as sd #python -m pip install sounddevice
from defs import *
import time, datetime, random, os.path, copy, mn, csv

# Add stuff at start of log file
def add_log(f, start, end):
    f = open(f,'r+')
    content = f.read()
    f.seek(0, 0)
    f.write(start.rstrip('\r\n') + '\n' + end.rstrip('\r\n') + '\nfp: ' + 
        ok_data[0] + '\nTask: ' + ok_data[1] +  '\n\n' + content)
    f.close()
# Pause def --------------------------------------------------------------------
def paus(pauses,t,block,pport = 223):
    ''' 
    Keyword arguments:
    pauses -- list of trialnumbers where there should be a pauses (list)
    t -- the current trialnumber (int)
    block -- list of blocknumbers, ex (0,0,0,1,1,1,2,2,2) (list)
    pport -- what trigger to send for pause (int)
    '''
    if t in pauses:
        if block[t] != block[len(block)-1]:
            paus_info.setText('Short break\n%i/%i blocks done.\nHit space when ready.'% (block[t]+1, max(block)+1))
            paus_info.draw()
            win.flip()
            event.waitKeys(keyList=['space'])
            p_port.setData(pport)
            core.wait(0.5)
            win.flip()
            p_port.setData(255)
            core.wait(2)
# Practice def -----------------------------------------------------------------
def practice():
    if ok_data[1] == 'Validation':
        fileName = os.path.join(fileDir+'\AAN_val_'+fps+'.txt')
        dataFile = open(fileName, 'w')
        start_time = 'Experiment start: ' + datetime.datetime.now().strftime("%y-%m-%d %H:%M")
        dataFile.write('fp\ttrial\tcond\tlevel\tsub_resp\taware\n')
    random.seed()
    ntp = 100 #number of trials practice (has to be divisible by 10)
    crit_list = list()
    control_list = list()
    catch_list = list()

    #Create trials for practice/validation
    cond = [1]*8 + [2] + [3]
    level = [slvl]*8 + [slvl+10] + [-999]
    trig = [254]*8 + [253] + [251]

    rand_cond = []
    rand_lvl = []
    rand_trig = []

    # makes the trials
    for i in range(int(ntp/10)): #100/10 = 10
        order = random.sample(range(len(cond)),len(cond))
        for trial in range(len(cond)):
            rand_cond += [cond[order[trial]]]
            rand_lvl += [level[order[trial]]]
            rand_trig += [trig[order[trial]]]

    #Make trial order and randomise it
    # --------------------------------------------------------------------------
    hit_ready.draw() # press space to start
    win.flip()
    event.waitKeys(keyList=['space'])
    sound1 = create_stimulus(condition = 1, sdur = 0.1, s_lvl = slvl, fade = 0.005, fs = 48000)
    # control
    sound2 = create_stimulus(condition = 2, sdur = 0.1, s_lvl = slvl+10, fade = 0.005, fs = 48000)
    # catch
    sound3 = create_stimulus(condition = 3, sdur = 0.1, s_lvl = -999, fade = 0.005, fs = 48000)
    sounds = [sound1,sound2,sound3]
    # --------------------------------------------------------------------------
    for t in range(ntp):
        fixation.draw()
        win.flip() # show cross
        # ----------------------------------------------------------------------
        fs = 48000
        send_triggers(rand_trig[t])
        core.wait(0.09)
        sd.play(sounds[rand_cond[t]-1], fs, mapping = ch4)
        time.sleep(0.555)
        # ----------------------------------------------------------------------
        check4esc()        
# display instructions
        sub_inst.draw() # show subjective rating
        win.flip()
# get subjective response
        aware_resp = None
        while aware_resp is None:
            sub_resp = event.waitKeys(keyList=['num_1','num_2','num_3','escape'])
            #sub_resp = random_key(2)
            for thisKey in sub_resp:
                if (thisKey == 'num_1'): #Heard nothing
                    aware_resp = 0
                elif  (thisKey == 'num_2'): #Heard weakly
                    aware_resp = 1
                elif  (thisKey == 'num_3'): #Heard clearly
                    aware_resp = 1
                elif (thisKey == 'escape'):
                    core.quit()
        if rand_cond[t] == 1:
            crit_list.append(aware_resp)  # store aware 0,1
        elif rand_cond[t] == 2:
            control_list.append(aware_resp)
        elif rand_cond[t] == 3:
            catch_list.append(aware_resp)

        if ok_data[1] == 'Validation': # if validation, make log
            #'fp\ttrial\tcond\tlevel\tsub_resp\taware\n'
            # i    i      i      f       s        i
            dataFile.write('%i\t%i\t%i\t%f\t%s\t%i\n'
                %(fp, t+1, rand_cond[t], rand_lvl[t], sub_resp[0], aware_resp))
    if ok_data[1] == 'Validation': # if validation, store timestamps and close.
        dataFile.close()
        end_time = 'Experiment end: ' + datetime.datetime.now().strftime("%y-%m-%d %H:%M")
        add_log(fileName, start_time, end_time)
    # give some output to user
        aware_crit = calc_aware(crit_list)
        aware_control = calc_aware(control_list)
        aware_catch = calc_aware(catch_list)
        print('Percent aware critical: %.1f\nPercent aware control: %.1f\nPercent aware catch: %.1f\n' % (aware_crit * 100, aware_control * 100, aware_catch * 100))
# Staircase def ----------------------------------------------------------------
def staircase():
    # create a logfile for calibration
    fileName = os.path.join(fileDir+'\AAN_cal_'+fps+'.txt')
    dataFile = open(fileName, 'w')
    start_time = 'Experiment start: ' + datetime.datetime.now().strftime("%y-%m-%d %H:%M")
    dataFile.write('fp\ttrial\tstair\tcond\tlevel\tsubj_resp\n')

    # number of critical trials (has to be divisible by 3
    nts = 120 # number of trials staircase
    
    #Create trials for calibration
    cond_first = [1]*8 + [3]
    trig_first = [254]*8 + [251]
    cond = [1]*9 + [3]
    trig = [254]*9 + [251]
    order = random.sample(range(len(cond_first)),len(cond_first))
    rand_cond = []
    rand_trig = []
    # makes first 9 trials
    for trial in range(len(cond_first)):
        rand_cond += [cond_first[order[trial]]]
        rand_trig += [trig_first[order[trial]]]

    # makes the next 110 trials
    for i in range(int(nts*0.1-1)): #(120 * 0.1) - 1
        order = random.sample(range(len(cond)),len(cond))
        for trial in range(len(cond)):
            rand_cond += [cond[order[trial]]]
            rand_trig += [trig[order[trial]]]

    rand_cond = [1] + rand_cond
    rand_trig = [254] + rand_trig
    counter = 0
    # Make staircase
    sub_list = list()
    alt_list = list()
    info={}
    info['startPoints']=[-60,-80,-100]
    info['nTrials']= int(nts*0.9/3.0) #108/3 = 36
    info['observer']='Subject: '
    # create the staircase handler
    stairs=[]
    for thisStart in info['startPoints']:
        thisInfo = copy.copy(info)
        thisInfo['thisStart']=thisStart
        thisStair = data.StairHandler(startVal=thisStart,
            extraInfo=thisInfo, nUp=1, nDown=1, nTrials= int(nts*0.9/3.0),
            minVal=-200,maxVal=0,stepSizes=[8,8,4,4,2,2,1],stepType='lin')
        stairs.append(thisStair)
    # Run staircase trial
    hit_ready.draw()
    win.flip()
    event.waitKeys(keyList=['space'])
    for trialN in range(info['nTrials']):
        random.shuffle(stairs)
        for thisStair in stairs:
            try:
                level = int(thisStair.next())
            except StopIteration:
                break
            print('\nTrial number %i'%(counter))
            print('Should be condition: 1')
            print('Current condition: %i' %(rand_cond[counter]))
            print('Current stair level: %i\n'%(level))
            fixation.draw()
            win.flip() # show cross
            #-------------------------------------------------------------------
            fs = 48000 
            sound = create_stimulus(condition = rand_cond[counter], sdur = 0.1, s_lvl = level, fade = 0.005, fs = 48000)
            send_triggers(rand_trig[counter])
            core.wait(0.09)
            sd.play(sound, fs, mapping = ch4)
            time.sleep(0.555)
            #-------------------------------------------------------------------
            check4esc()
    # display instructions and wait
            sub_inst.draw()
            win.flip()

    # get subjective response
            aware_resp = None
            while aware_resp is None:
                sub_resp = event.waitKeys(keyList=['num_1','num_2','num_3','escape'])
                #sub_resp = random_key(2)
                for thisKey in sub_resp:
                    if (thisKey == 'num_1'): #Heard nothing
                        aware_resp = 0
                    elif  (thisKey == 'num_2'): #Heard weakly
                        aware_resp = 1
                    elif  (thisKey == 'num_3'): #Heard clearly
                        aware_resp = 1
                    elif (thisKey == 'escape'):
                        core.quit()

            dataFile.write('%i\t%i\t%i\t%i\t%f\t%i\n'%(fp, trialN+1, thisStair.extraInfo['thisStart'], rand_cond[counter], level, int(aware_resp)))
            counter += 1
            # CATCH TRIALS
            if counter != nts:
                while rand_cond[counter] == 3:
                    print('\nTrial number %i'%(counter))
                    print('Should be condition: 1')
                    print('Current condition: %i' %(rand_cond[counter]))
                    print('Current stair level: %i\n'%(level))
                    fixation.draw()
                    win.flip() # show cross
                    #-------------------------------------------------------------------
                    fs = 48000
                    sound = create_stimulus(condition = rand_cond[counter], sdur = 0.1, s_lvl = level, fade = 0.005, fs = 48000)
                    send_triggers(rand_trig[counter])
                    core.wait(0.09)
                    sd.play(sound, fs, mapping = ch4)
                    time.sleep(0.555)
                    #time.sleep(len(sound) / float(fs))
                    #-------------------------------------------------------------------
                    check4esc()
            # display instructions and wait
                    sub_inst.draw()
                    win.flip()
            # get subjective response
                    alt_resp = None
                    while alt_resp is None:
                        alt_resp = event.waitKeys(keyList=['num_1','num_2','num_3','escape'])
                        #alt_resp = random_key(2)
                        for thisKey in alt_resp:
                            if (thisKey == 'num_1'): #Heard nothing
                                alt_resp = 0
                            elif  (thisKey == 'num_2'): #Heard weakly
                                alt_resp = 1
                            elif  (thisKey == 'num_3'): #Heard clearly
                                alt_resp = 1
                            elif (thisKey == 'escape'):
                                core.quit()
                    alt_list.append(alt_resp)
                    dataFile.write('%i\t%i\t%i\t%i\t%f\t%i\n'%(fp, trialN+1, thisStair.extraInfo['thisStart'], rand_cond[counter], level, int(alt_resp)))
                    if counter == nts-1:
                        print('It broke out of loop')
                        break
                    else:
                        counter += 1
            print('This response is added to stair: %i' %(aware_resp))
            # add the data to the staircase so it can calculate the next level
            thisStair.addResponse(aware_resp)
            # calc chance
            sub_list.append(aware_resp)  # store aware 0,1
    for thisStair in stairs:
        dataFile.write('%.1f\t%d\n' %(np.average(thisStair.reversalIntensities[-6:]),thisStair.extraInfo['thisStart']))
    # staircase has ended
    dataFile.close()
    end_time = 'Experiment end: ' + datetime.datetime.now().strftime("%y-%m-%d %H:%M")
    add_log(fileName, start_time, end_time)
    print('reversals:')
    for thisStair in stairs:
        print('mean 6 final reversals = %.1f for stair %d' %(np.average(thisStair.reversalIntensities[-6:]),thisStair.extraInfo['thisStart']))
    print('mean false alarm rate = %f' %(np.mean(alt_list)))
# Experiment def ---------------------------------------------------------------
def experiment():
    '''
    Conditions:
    1 = critical (calibrated dB)
    2 = control (+10 dB)
    3 = catch (~0 dB)
    '''
    fileName = os.path.join(fileDir+'\AAN_fp'+fps+'.txt')
    dataFile = open(fileName, 'w')
    start_time = 'Experiment start: ' + datetime.datetime.now().strftime("%y-%m-%d %H:%M")
    dataFile.write('fp\ttrial\tblock\tcond\tlevel\tsubj_resp\taware\ttrigger\n')

    random.seed()
    nte = 800 #600 number of trials experiment
    nbl = 8 #8 number of blocks
    # Blocks and breaks 600 / 8 = 75
    block = []
    pauses = []
    for i in range(nbl):
        pauses.append([])
        pauses[i] = int(nte/nbl * i + (nte/nbl - 1))
        block += [i] * int(nte/nbl)

    # Create trials for experiment
    cond = [1]*8 + [2] + [3]
    level = [slvl]*8 + [slvl+10] + [-999]
    trig = [254]*8 + [253] + [251]

    rand_cond = []
    rand_lvl = []
    rand_trig = []

    # makes the trials
    for i in range(int(nte/10)): # 800/10 = 80
        order = random.sample(range(len(cond)),len(cond))
        for trial in range(len(cond)):
            rand_cond += [cond[order[trial]]]
            rand_lvl += [level[order[trial]]]
            rand_trig += [trig[order[trial]]]
    
    #---------------------------------------------------------------------------
    # WARM UP SYSTEM WARM UP SYSTEM WARM UP SYSTEM WARM UP SYSTEM WARM UP SYSTEM
    fs = 48000
    sound = create_stimulus(condition = 3, sdur = 0.1, s_lvl = -999, fade = 0.005, fs = 48000)
    sd.play(sound, fs, mapping = ch4)
    time.sleep(0.5)
    # critical
    sound1 = create_stimulus(condition = 1, sdur = 0.1, s_lvl = slvl, fade = 0.005, fs = 48000)
    # control
    sound2 = create_stimulus(condition = 2, sdur = 0.1, s_lvl = slvl+10, fade = 0.005, fs = 48000)
    # catch
    sound3 = create_stimulus(condition = 3, sdur = 0.1, s_lvl = -999, fade = 0.005, fs = 48000)
    sounds = [sound1,sound2,sound3]
    #---------------------------------------------------------------------------
    exp_instruction.draw()
    win.flip()
    event.waitKeys(keyList=['space'])
    send_triggers(223) # start block
    for t in range(len(rand_cond)):
        fixation.draw()
        win.flip() # show cross
        #-----------------------------------------------------------------------
        fs = 48000
        send_triggers(rand_trig[t])
        core.wait(0.09)
        sd.play(sounds[rand_cond[t]-1], fs, mapping = ch4)
        time.sleep(0.555)
        #-----------------------------------------------------------------------
        check4esc()

# display instructions and wait
        sub_inst.draw()
        win.flip()

# get subjective response
        aware_resp = None
        while aware_resp is None:
            sub_resp = event.waitKeys(keyList=['num_1','num_2','num_3','escape'])
            #sub_resp = random_key(2)
            for thisKey in sub_resp:
                if (thisKey == 'num_1'): #Heard nothing
                    aware_resp = 0
                elif  (thisKey == 'num_2'): #Heard weakly
                    aware_resp = 1
                elif  (thisKey == 'num_3'): #Heard clearly
                    aware_resp = 1
                elif (thisKey == 'escape'):
                    core.quit()
        #fp\ttrial\tblock\tcond\tlevel\tsubj_resp\taware\ttrigger\n
        dataFile.write('%i\t%i\t%i\t%i\t%f\t%s\t%i\t%i\n'
            % (fp,t+1,block[t]+1,rand_cond[t], rand_lvl[t], 
            sub_resp[0], aware_resp, rand_trig[t]))
        paus(pauses,t,block,pport = 223)
    dataFile.close()
    end_time = 'Experiment end: ' + datetime.datetime.now().strftime("%y-%m-%d %H:%M")
    add_log(fileName, start_time, end_time)
# ------------------------------------------------------------------------------
sd.default.device = "ASIO Fireface USB"
print 'Sound device ------------------------------------------------------------'
print sd.default.device
#print sd.query_devices()
print '-------------------------------------------------------------------------'

#Parallel port
#p_port = parallel.ParallelPort(address = "0x4020") # room 225
p_port = parallel.ParallelPort(address='0x3028') # room 223

#nr channels
ch2 = [1,2]
ch4 = [3,4,1,2]
memory_data = []

if os.path.isfile('memory.csv'):
    print('Found memory!')
    with open('memory.csv', 'rb') as csvfile:
        reader = csv.reader(csvfile, delimiter=',')
        for i in reader:
            memory_data += i
    expInfo = gui.Dlg(title="Auditory detection")
    expInfo.addField('Subject', memory_data[0])
    expInfo.addText('Experiment Info')
    expInfo.addField('Task', choices = ['Validation','Practice','Staircase','Experiment'])
    expInfo.addField('Level', float(memory_data[1]))
    ok_data = expInfo.show()  # show dialog and wait for OK or Cancel
    if expInfo.OK:  # or if ok_data is not None
        if ok_data[2] > 0:
            print('Too high level!')
            core.quit()
        print('Subject: %i\nTask: %s\nSignal level: %f\n' %(int(ok_data[0]), ok_data[1], ok_data[2]))
    else:
        print('user cancelled')
        core.quit()
else:
    print('No memory, making.')
    # present a dialogue to give subject nr and select program
    expInfo = gui.Dlg(title="Auditory detection")
    expInfo.addField('Subject', '01')
    expInfo.addText('Experiment Info')
    expInfo.addField('Task', choices = ['Validation','Practice','Staircase','Experiment'])
    expInfo.addField('Level', -40)
    ok_data = expInfo.show()  # show dialog and wait for OK or Cancel
    if expInfo.OK:  # or if ok_data is not None
        if ok_data[2] > 0:
            print('Too high level!')
            core.quit()
        print('Subject: %i\nTask: %s\nSignal level: %f\n' %(int(ok_data[0]), ok_data[1], ok_data[2]))
    else:
        print('user cancelled')
        core.quit()

# ------------------------------------------------------------------------------
# SET GLOBALS    
fp = int(ok_data[0])
fps = ok_data[0]
slvl = ok_data[2]

with open('memory.csv', 'wb') as csvfile:
    writer = csv.writer(csvfile, delimiter=',')
    writer.writerow([fps, str(slvl)])

# set file directory
fileDir = 'C:\Users\SWIENSLAB01\Desktop\AAN\data' # room 223
#fileDir = 'C:\Users\SWIENSLAB01\Dropbox\Rasmus_Eklund\RasmusPhD\AAN\PythonExperiment\data' # room 223
#fileDir = 'C:\Users\RAEK2031\Dropbox\Rasmus_Eklund\RasmusPhD\AAN\PythonExperiment\data' # work computer
#fileDir = 'C:\Users\SWIENSLAB03\Dropbox\Rasmus_Eklund\RasmusPhD\AAN\PythonExperiment\data' # room 225
#fileDir = 'F:\Dropbox\Rasmus_Eklund\RasmusPhD\AAN\PythonExperiment\data' # home
#fileDir = 'C:\Users\SWIENSLAB02\Dropbox\Rasmus_Eklund\RasmusPhD\AAN\PythonExperiment\data' # control room

# create window and stimuli
globalClock = core.Clock()  # to keep track of time
trialClock = core.Clock()  # to keep track of time

#Screen 1920, 1080
win = visual.Window([1920, 1080], allowGUI = False, fullscr = False, #1920, 1080
    monitor = 'testMonitor', units = 'height')

#visual stimuli
fixation = visual.TextStim(win, color='black', height = 0.03, text = '+')
sub_inst = visual.TextStim(win, color='black', height = 0.02,
    text='1. I did not hear any stimulus\n\n2. I heard the stimulus weakly\n\n3. I heard the stimulus clearly')
exp_instruction = visual.TextStim(win, color='black', height = 0.02,
    text = 'Listen for a short tone on every trial.\n\nRate your auditory experience with the 1, 2, and 3 buttons.\n\nThe experiment is divided into eight blocks. \n\nAfter each block there will be time for a short break. \n\nTry to avoid blinking while the sounds are played. Eyeblinks make artifacts in the EEG data.\n\n\nHit space when you are ready to continue!')
hit_ready = visual.TextStim(win, color='black', height = 0.02,
    text = 'Hit space when you are ready to start!')
end = visual.TextStim(win, color='black', height = 0.02,
    text = 'Done with '+ ok_data[1]+'!\nCall the experimenter with the door bell!')
paus_info = visual.TextStim(win, color='black', height = 0.02,
    text = '', pos = (0,-0.2))

#==============================================================================#
# Procedure
if ok_data[1] == 'Experiment':
    experiment()
if ok_data[1] == 'Staircase':
    staircase()
if ok_data[1] == 'Validation':
    practice()
if ok_data[1] == 'Practice':
    practice()

end.draw()
win.flip()
event.waitKeys()

win.close()
core.quit()