#!/usr/bin/python
# -*- coding: utf-8 -*-

# Function for creating stimulus for Rasmus Eklund's experiment)
# Created: MN-2017-10-10; Revised: RE-2017-10-12

import mn, random
from psychopy import core, visual, gui, data, event, parallel
import numpy as np

#Parallel port
#p_port = parallel.ParallelPort(address = "0x4020") # room 225
p_port = parallel.ParallelPort(address='0x3028') # room 223
def random_key(key):
    '''Randomise a response.
    Keyword arguments:
    key -- 1 or 2, 1 = obj resp, 2 = sub resp
    '''
    obj_key = ['delete','pagedown']
    sub_key = ['1','2','3']
    if key == 1:
        resp = random.sample(obj_key,1)
    elif key == 2:
        resp = random.sample(sub_key,1)
    return resp

def send_triggers(in_trig):
    p_port.setData(in_trig)
    core.wait(0.05)
    p_port.setData(255)

def check4esc():
    ''' Checks for escape.
    If escape is pressed, it terminates the program.
    '''
    keys = event.getKeys(keyList = ['escape'])
    if len(keys)>0 and 'escape' in keys:
        core.quit()

# def paus(win,pauses,trialnumber,blocks,pport = 192):
    # ''' 
    # Keyword arguments:
    # pauses -- list of trialnumbers where there should be a pauses (list)
    # trialnumber -- the trialnumber (int)
    # blocks -- list of blocknumbers, ex (0,0,0,1,1,1,2,2,2) (list)
    # pport -- what trigger to send for pause (int)
    # '''
    # if trialnumber in pauses:
        # check4esc()
        # blockNr=blocks[trialnumber]+1
        # paus_info.setText('Short  break\n%i/\n%i blocks done.\nHit space when ready.'% blockNr, max(blocks))
        # paus_info.draw()
        # win.flip()
        # event.waitKeys(keyList=['space'])
        # if block[t] !=4:
            # p_port.setData(pport)
            # win.flip()
            # p_port.setData(255)
        # core.wait(2)

# AAN defs --------------------------------------------------------------------#

def calc_aware(sub_list):
    '''
    Keyword arguments:
    sub_list -- list of numbers: 0 = unaware, 1 = aware

    Return:
    aware -- (aware/(aware + unaware)) (float)
    '''

    aware_index = []
    # find and enumerate all reported aware (sub_list)
    for i, x in enumerate(sub_list):
        if x == 1:
            aware_index += [i]

    # calculate percent reported aware
    aware = sum(sub_list)*1.0/len(sub_list)

    return aware

def calc(obj_list, sub_list):
    '''
    Calculates percentage of trials:
        reported as aware, correct, aware and correct, unaware and correct
    
    Keyword arguments:
    obj_list -- list of numbers: 0 = incorrect, 1 = correct
    sub_list -- list of numbers: 0 = unaware, 1 = aware
    
    Return:
    aware -- (aware/(aware + unaware)) (float)
    correct -- correct/(correct + incorrect))
    aware_correct -- (correct aware/(correct aware + correct unaware))
    unaware_correct -- (correct unaware/(correct aware + correct unaware))
    '''
    aware_index = []
    unaware_index = []
    aware_corr_index = []
    unaware_corr_index = []

    # find and enumerate all reported aware (sub_list)
    for i, x in enumerate(sub_list):
        if x == 1:
            aware_index += [i]

    # find the corresponding correct and incorrect responses (obj_list)
    for i in aware_index:
        aware_corr_index += [obj_list[i]]

    # find and enumerate all reported unaware (sub_list)
    for i, x in enumerate(sub_list):
        if x == 0:
            unaware_index += [i]

    # find the corresponding correct and incorrect responses (obj_list)
    for i in unaware_index:
        unaware_corr_index += [obj_list[i]]

    # calculate percent reported aware
    aware = sum(sub_list)*1.0/len(sub_list)
    # calculate percent correct
    correct = sum(obj_list)*1.0/len(obj_list)
    # calulate percent correct when reported aware
    if sum(aware_corr_index) != 0:
        aware_correct = sum(aware_corr_index)*1.0/len(aware_corr_index)
    else:
        aware_correct = 0
    # calculate percent correct when reported not aware
    if sum(unaware_corr_index) != 0:
        unaware_correct = sum(unaware_corr_index)*1.0/len(unaware_corr_index)
    else:
        unaware_correct = 0
    return aware, correct, aware_correct, unaware_correct
    
    
def create_stimulus(condition = 1, sdur = 0.1, s_lvl = 0.0, fade = 0.05, fs = 48000):
    ''' Creates stimulus for Rasmus experiment (add more info here.....)
    
    Keyword arguments:
    condition -- make tone or catch: 1 = tone, 2 = catch (int)
    stype -- Type of signal: 1 = sinusoid, 2 = square pulse (int)
    sdur -- duration of signal in seconds (float)
    s_lvl -- Level of signal relative to whote noise, rms dB (float)
    fade -- Duration of fade-in and fade-out, in seconds (float)
    fs -- Sampling frequncy (int)

    Return:
    stimulus -- stereo * 2 signal (4xN numpy array)
    '''

    # Create signal ------------------------------------------------------------
    s = mn.create_sinusoid (freq = 1000, phase = 0, fs = fs, dur = sdur)
    # --------------------------------------------------------------------------

    # Add zeros to signal to make it as long as the white noise ----------------
    s_stimtracker = mn.set_gain(s, -20) # make signal at higher dB without white noice for stimtracker
    s = mn.set_gain(s, s_lvl) # make signal at calibrated lvl
    catch = np.zeros(int(sdur * fs))
    # --------------------------------------------------------------------------

    # Combine noise and signal into stereo -------------------------------------
    fadesamples = int(fade * fs)
    if (condition == 1 or condition == 2): # critical and control trials
        ch1 = mn.fade(s, samples = fadesamples)
        ch2 = mn.fade(s, samples = fadesamples)
        ch3 = mn.fade(catch, samples = fadesamples)
        ch4 = mn.fade(s_stimtracker, samples = fadesamples)
    elif (condition == 3): # catch trials
        ch1 = mn.fade(catch, samples = fadesamples)
        ch2 = mn.fade(catch, samples = fadesamples)
        ch3 = mn.fade(catch, samples = fadesamples)
        ch4 = mn.fade(s_stimtracker, samples = fadesamples)

    stimulus = np.transpose(np.array([ch1,ch2,ch3,ch4]))
    # --------------------------------------------------------------------------
    return stimulus